from transcription import *
import subprocess
from sets import Set
import operator
import os

# Extract the top ASR / Semantic hypotheses from the xml
# Returns two dictionaries, first with hyps, second with refs
def extractTopHypFromXml(test, hyptag='asr', trantag='transcription'):
    hyp = {}
    ref = {}
    
    uturns = getList('userturn', test.content)
        
    if len(uturns) == 0:
        # Try with uturns in turn tags
        turns = getList('turn', test.content)
        for turn in turns:
            uturns = uturns + (getList('userturn', turn))

    for uturn in uturns:
        rec = getList('rec', uturn)  # Note - this shouldn't be a list but just allow that in case we have xml errors
        asr = getList(hyptag, uturn)
        tran = getList(trantag, uturn)
        if len(asr) > 0 and len(rec) > 0 and len(tran) > 0:
            asr = asr[0]
            rec = rec[0]
            tran = tran[0]
            asrhyp = getList(hyptag+'hyp', asr)
            asrhyp = getAsrHyps(asrhyp)

            if "fname" in rec and "value" in tran and len(asrhyp) > 0:
                r = rec["fname"]
                t = tran["value"]

                orderedAsr = sorted(asrhyp.iteritems(), key=operator.itemgetter(1), reverse=True)
                topAsr = orderedAsr[0][0]
                topProb = orderedAsr[0][1]
                hyp[r] = t
                ref[r] = topAsr
                
    return hyp, ref

# Extract all the top values from the corpus into dictionaries from rec name to asr, tran, semi, uact
def extractValuesFromCorpus(xmlfiles):
    asr = {}
    tran = {}
    semi = {}
    uact = {}
    
    for inputDialogue in xmlfiles:
        test = DialogTranscription(inputDialogue)
        newAsr, newTran = extractTopHypFromXml(test, 'asr', 'transcription')        
        newSemi, newUact = extractTopHypFromXml(test, 'semi', 'uact')
        for r in newAsr.keys():
            if not r in newSemi.keys():
                newAsr.pop(r)
        for r in newSemi.keys():
            if not r in newAsr.keys():
                newSemi.pop(r)
        
        asr.update(newAsr)
        tran.update(newTran)
        semi.update(newSemi)
        uact.update(newUact)
        
    return asr, tran, semi, uact
