# Testing program
# Place your matrices in the predictions folder with names in the format:
#       name1.types.mat and name2.items.mat
# This program computes the type and item error

import os
from scipy.sparse import lil_matrix, coo_matrix
from scipy.io import loadmat, savemat
from scipy.io import mmread, mmwrite
import numpy

def loadMatrices(predictionFile):
    items = None
    predictionItems = predictionFile + ".items"
    if os.path.isfile(predictionItems + ".mat"):
        predictionItems += ".mat"
        items = loadmat(predictionItems)['Y']
    elif os.path.isfile(predictionItems + ".mtx"):
        predictionItems += ".mtx"
        items = mmread(predictionItems)
    else:
        print "Error - no item file for " + predictionFile
        
    types = None
    predictionTypes = predictionFile + ".types"
    if os.path.isfile(predictionTypes + ".mat"):
        predictionTypes += ".mat"
        types = loadmat(predictionTypes)['Z']
    elif os.path.isfile(predictionTypes + ".mtx"):
        predictionTypes += ".mtx"
        types = mmread(predictionTypes)
    else:
        print "Error - no item file for " + predictionFile
        
    return items, types

def score(predictionFile, refFile):
    pItems, pTypes = loadMatrices(predictionFile)
    rItems, rTypes = loadMatrices(refFile)
    N = pItems.shape[0]
    if pItems.shape[1] < rItems.shape[1]:
        y = rItems.shape[1]
        A = coo_matrix(pItems)
        pItems = coo_matrix((A.data,(A.row, A.col)), shape=(N,y))
    difItems = abs(pItems - rItems)
    sumDif = difItems.sum()
    numItems = rItems.sum()
    
    equal = len([t1 for (t1, t2) in zip(rTypes.flatten(), pTypes.flatten()) if t1 == t2])
    
    print "Scoring " + predictionFile + ", ItemDif = " + str(sumDif) + ", ItemErrorRate = " + str(sumDif / float(numItems)) + ", TypesEqual = " + str(equal) + ", TypeError = " + str(1 - (equal / float(N)))
    

if __name__ == '__main__':
    preprocessed = "preprocessed"
    
    uactFile = os.path.join(preprocessed, "uact-test") # Semantic transcriptions
    semiFile = os.path.join(preprocessed, "semi-test") # Top semantic hypothesis

    score(semiFile, uactFile)
    
    files = os.listdir('predictions')
    names = ([f.split('.')[0] for f in files if len(f) > 0])
    names = set([f for f in names if len(f) > 0])
    for name in names:
        score(os.path.join('predictions', name), uactFile)
    